#!/usr/bin/env python
# -*- coding: UTF-8 -*-
'''
  File Name : moo_grideye_i2c.py

  ROBOBA041 : I2C インターフェース AMG8832（GridEYE 8X8 赤外線アレイセンサ）操作クラス

  Ver.0.1 : 2016/10 初版 クラス化

  開発環境 : Raspberry Pi
             Python 3.4.2
             Python 2.7.9

  CopyLeft 2016 Moo Soft (Akiyosi Yokoyama)
  
  参考 URL
    i2c python documentation
      http://wiki.erazor-zone.de/wiki:linux:python:smbus:doc
    Grid-EYE
      http://industrial.panasonic.com/jp/products/sensors/built-in-sensors/grid-eye
'''
import six
import os
import time
import smbus
from threading import Thread

# ============================================== #
# GridEYE (8X8 赤外線アレイセンサ) 操作クラス
# ============================================== #
class GridEyeI2C(object):

    # ========================================== #
    # コンストラクタ
    # addr  : 0x69 GridEYE I2C スレーブアドレス(or 0x68)
    # ch    : SMBus チャネル
    # debug : デバッグモードフラグ
    # ========================================== #
    def __init__(self, addr=0x69, ch=1, debug=False):
        self.addr       = addr  # 0x69 GridEYE I2C スレーブアドレス(or 0x68)
        self.debug      = debug # デバッグモードフラグ
        self.PCLT       = 0x00  # パワーコントロールレジスタ
        self.PCLT_NOMAL = 0x00  # ノーマルモード
        self.RST        = 0x01  # リセット
        self.RST_INIT   = 0x3f  # イニシャルリセット
        self.BUS        = None  # SMBus インスタンス
        self.eye_init(ch)       # Grid Eye 初期化

    # ============================================== #
    # Grid Eye 初期化
    # 戻り値 : 無し
    #
    # BUS        : SMBus インスタンス
    # PCLT       : パワーコントロールレジスタ
    # PCLT_NOMAL : ノーマルモード
    # RST        : リセット
    # RST_INIT   : イニシャルリセット
    # ============================================== #
    def eye_init(self, ch):
        try:
            self.BUS = smbus.SMBus(ch)                 # SMBus インスタンス
            self.i2c_write(self.RST, self.RST_INIT)    # イニシャルリセット
            time.sleep(1.0)
            self.i2c_write(self.PCLT, self.PCLT_NOMAL) # パワーコントロールノーマル
            time.sleep(0.1)
        except Exception as e:
            self.BUS = None
            print('eye_init:{0}'.format(e))

    # ============================================== #
    # 1バイト書込み
    # waddr  : アドレス
    # value  : データ
    # 戻り値 : 無し
    #
    # self.BUS  : SMBus インスタンス
    # self.addr : Grid-EYE アドレス
    # ============================================== #
    def i2c_write(self, waddr, value):
        try:
            self.BUS.write_byte_data(self.addr, waddr, value)
        except Exception as e:
            print('i2c_write:{0}'.format(e))

    # ============================================== #
    # 1バイト読込み
    # raddr  : 読込み開始アドレス
    # 戻り値 : 読込んだデータ
    #
    # self.BUS  : SMBus インスタンス
    # self.addr : Grid-EYE アドレス
    # ============================================== #
    def read_byte(self, raddr):
        result = -1
        try:
            result = self.BUS.read_byte_data(self.addr, raddr)
            if self.debug:                                     # デバッグモード
                print('Address   : 0x{0:0x}\nRead data : 0x{1:0x}'.format(raddr, result))
        except Exception as e:
            print('read_byte:{0}'.format(e))
        return result

    # ============================================== #
    # ブロック読込み(最大32バイト)
    # raddr  : 読込み開始アドレス
    # rbyte  : 読込バイト数
    # 戻り値 : 読込んだデータのリスト(long[])
    #
    # self.BUS  : SMBus インスタンス
    # self.addr : Grid-EYE アドレス
    # ============================================== #
    def read_block(self, raddr, rbyte = 32):
        result = []
        try:
            rbyte = 32 if rbyte > 32 else rbyte
            rbyte =  1 if rbyte <  1 else rbyte
            result = self.BUS.read_i2c_block_data(self.addr, raddr, rbyte)
            if self.debug:                           # デバッグモード
                tmp = ''
                for c in result:                     # 16進文字列作成
                    tmp += '{0:x}'.format(c).strip()
                print('Address   : 0x{0:x}\nRead byte : 0x{1:x}\nRead data : {2}'.format(raddr, rbyte, tmp))
        except Exception as e:
            print('read_block:{0}'.format(e))
        return result

    # ============================================== #
    # 温度レジスタ温度摂氏計算 温度データは、12bit
    # templist : 温度データリスト 2の補数形式
    # 戻り値   : 温度データ文字列
    #            t_max  : 最高温度
    #            t_show : ' ' で区切られ 8温度データ毎に改行付加(画面表示用)
    #            t_find : 最高温度位置(行)検索用
    # ============================================== #
    def calc_temp(self, templist):
        t_show = ''          # 温度画面表示用
        t_find = ''          # 最高温度位置(行)検索用
        lsbval = 0.25        # 1LSB が相当する温度 0.25℃
        f_str  = '{0:+3.2f}' # 温度フォーマット文字列
        t_max  = -999        # 最高温度
        try:
            for c in range(0, len(templist) // 2):
                tool = '{0:08b}'.format(templist[c * 2]    , 'b')       # 8桁の2進数に変換
                tooh = '{0:08b}'.format(templist[c * 2 + 1], 'b')
                tooh = tooh[4 : 8]                                      # 下位4ビット取得
                temp = int(tooh + tool, 2)                              # 温度取得
                if tooh[0] == '1': temp = -(4096 - temp)                # ビットが立っていればマイナス(2の補数を得る)
                temp *= lsbval                                          # ℃換算
                if t_max < temp: t_max = temp                           # 最高温度
                tmp = f_str.format(temp)                                # 画像表示用温度文字列作成
                t_find += tmp + ','                                     # 最高温度位置(行)検索用
                tmp = f_str.format(temp).rjust(7, ' ')                  # 温度文字列作成、桁整形
                if c % 8 == 0:
                   if c == 0: t_show = tmp                              # 最初のデータは温度データのみ
                   else:      t_show += '\n' + tmp                      # 温度レジスタの画素配列に注意
                else:         t_show += ' '  + tmp
            t_show = t_show if t_show.endswith('\n') else t_show + '\n' # 最後に '\n' 付加
            t_find = t_find[: -1] if t_find.endswith(',') else t_find   # 最後の ',' 削除
        except Exception as e:
            print('calc_temp:{0}:{1}'.format(type(e), e))
        return t_max, t_show, t_find

    # ============================================== #
    # 温度レジスタからデータ取得
    # 戻り値 : tmax   : 最高温度
    #          trow   : 最高温度行位置(0-7)
    #          tcol   : 最高温度列位置(0-7)
    #          result : ' ' で区切られ 8温度データ毎に改行付加(画面表示用)
    # 温度データアドレス(0x80 ～ 0xff)
    # ============================================== #
    def get_temp(self):
        result = ''
        tmax  = -999; tcol  = 7; tmax0 = -999; tcol0 = 7
        tmax1 = -999; tcol1 = 7; tmax2 = -999; tcol2 = 7
        tmax3 = -999; tcol3 = 7; trow = 0
        try:
            t0 = self.read_block(0x80)                          # 0x80 から 32バイト読込
            t1 = self.read_block(0xa0)                          # 0xa0 から 32バイト読込
            t2 = self.read_block(0xc0)                          # 0xc0 から 32バイト読込
            t3 = self.read_block(0xe0)                          # 0xe0 から 32バイト読込
            tmax0, d0, d_0 = self.calc_temp(t0)                 # 温度計算
            tmax1, d1, d_1 = self.calc_temp(t1)
            tmax2, d2, d_2 = self.calc_temp(t2)
            tmax3, d3, d_3 = self.calc_temp(t3)
            tmax = max(tmax0, tmax1, tmax2, tmax3)              # 最高温度
            result = d0 + d1 + d2 + d3                          # ' ' で区切られ 8温度データ毎に改行付加(画面表示用)
            tmp  = '{0},{1},{2},{3}'.format(d_0, d_1, d_2, d_3) # 最高温度位置(行)検索用文字列
            smax = '{0:+3.2f}'.format(tmax)
            tmps = tmp.split(',')
            trow = tmps.index(smax) // 8                        # 最高温度行位置
            tcol = tmps.index(smax) %  8                        # 最高温度列位置
        except Exception as e:
            print('get_temp:{0}'.format(e))
        return tmax, trow, tcol, result

    # ============================================== #
    # ファイル書き込みスレッド起動
    # fname  : ファイル名
    # wData  : 書き込みデータ
    # mode   : ファイルモード 'w':上書き 'a':追加
    # 戻り値 : 無し
    # ============================================== #
    def write_text(self, fname, wData, mode='w'):
        try:
            th = Thread(target=self.write_textThread, args=(fname, wData, mode))
            th.start()
        except Exception as e:
            print('writeText:{0}'.format(e))

    # ============================================== #
    # ファイル書き込みスレッド
    # f_name : ファイル名
    # wData  : 書き込みデータ
    # mode   : ファイルモード 'w':上書き 'a':追加
    # 戻り値 : 無し
    # ============================================== #
    def write_textThread(self, fname, wData, mode='w'):
        try:
            if six.PY2:
                with open(fname, mode) as f:
                    f.write(wData)
                    f.flush()
            if six.PY3:
                with open(fname, mode, encoding='utf-8') as f:
                    f.write(wData)
                    f.flush()
        except Exception as e:
            print('write_textThread:{0}'.format(e))

# end class GridEyeI2C

# ============================================== #
# メインループ
# ============================================== #
def loop():
    try:
        gr = GridEyeI2C(addr=0x69, debug=True) # Grid Eye 操作クラス
        while True:
            time.sleep(1.0)
            tmax, trow, tcol, result = gr.get_temp()
            smax = '{0:+3.2f}'.format(tmax)
            gr.write_text('a.txt', '{0},{1},{2}'.format(trow, tcol, smax))
            print('Max:{0} Row:{1} Col:{2}\n{3}'.format(smax, trow, tcol, result))
    except Exception as e:
        print('loop:{0}'.format(e))

# ============================================== #
if __name__=='__main__':
    loop()
