#!/usr/bin/env python
# -*- coding: UTF-8 -*-
'''
  File Name : moo_moveservo.py

  サーボ駆動クラス

  Ver.0.1 : 2016/10 初版

  開発環境 : Raspberry Pi
             Python 3.4.2
             Python 2.7.9

  CopyLeft 2016 Moo Soft (Akiyosi Yokoyama)

  参考 URL     : http://akizukidenshi.com/catalog/g/gM-08761/
  データシート : http://akizukidenshi.com/download/ds/towerpro/SG90.pdf
'''
import six
import os
import time
from threading import Thread
from moo_servo import MooServo # サーボモーター制御クラス

# ============================================== #
# サーボ駆動クラス
# ============================================== #
class MoveServo(Thread):

    # ========================================== #
    # コンストラクタ
    # fwait : 位置情報ファイル監視インターバル
    # fname : 位置情報ファイル名
    # addr  : 0x4f PCA9685 アドレス
    # frq   : 50 RDS3115MG, Micro Servo 9g : 50 Hz Freq:40～1000
    # ========================================== #
    def __init__(self, fwait=1.0, fname='a.txt', addr=0x4f, frq=50):
        Thread.__init__(self)
        self.daemon = True
        self.fwait  = fwait
        self.fname  = fname
        # サーボ関連
        self.s0_ch     = 0                                # Tilt サーボ制御チャネル(0-7)
        self.s1_ch     = 1                                # Pan  サーボ制御チャネル(0-7)
        self.servo     = MooServo(addr, frq)              # サーボモーター制御クラス
        
        s_Min    = self.servo.get_pulse(0.5, frq, 'SG90 9g Micro Servo : 0.5ms ->   0')
        s_Center = self.servo.get_pulse(1.5, frq, 'SG90 9g Micro Servo : 1.5ms ->  90')
        s_Max    = self.servo.get_pulse(2.4, frq, 'SG90 9g Micro Servo : 2.4ms -> 180')
        self.s0_Min    = s_Min + 11
        self.s0_Center = s_Center
        self.s0_Max    = s_Max - 11
        self.s1_Min    = s_Min + 11
        self.s1_Center = s_Center
        self.s1_Max    = s_Max - 11
        self.s0_now    = self.s0_Center                   # サボ現在位置
        self.s1_now    = self.s1_Center                   # サボ現在位置
        self.s0_step   = (self.s0_Max - self.s0_Min) // 8 # 行、列移動単位
        self.s1_step   = (self.s1_Max - self.s1_Min) // 8 # 行、列移動単位
        self.m_wait    = 0.003                            # サーボが値1移動時の待ち時間
        self.servo.servo_between(self.s0_ch, self.s0_now, self.s0_Center, self.s0_Min, self.s0_Max, self.m_wait)
        self.servo.servo_between(self.s1_ch, self.s1_now, self.s1_Center, self.s1_Min, self.s1_Max, self.m_wait)

    # ========================================== #
    # メイン処理
    # ========================================== #
    def run(self):
        try:
            self.write_text(self.fname, '', 'w')                 # 開始時にファイルがあれば中身を空にする
            while True:
                try:
                    time.sleep(self.fwait)                       # ファイルチェック間隔
                    if os.path.exists(self.fname):               # os.pathisfile()
                        fst = self.read_text(self.fname).strip() # ファイルの内容読込 e.g : 3,6,32.65 最高温度位置 -> 3行,6桁,32.65℃
                        row, col, temp = self.cmd_check(fst)     # 文字列チェック
                        self.write_text(self.fname, '', 'w')     # ファイルの中身を空にする
                        if row < 0 or col < 0 or temp < -99: continue

                        if   row < 3: row = 3                    # Tilt 稼動範囲制御 (必要なら)
                        elif row > 5: row = 5
                        row = row * self.s0_step + self.s0_Min
                        self.move_servo(self.s0_ch, row)         # Tilt サーボ制御
                        col = col * self.s1_step + self.s1_Min
                        print('-' * 10)
                        print('row : {0}'.format(row))
                        print('col : {0}'.format(col))
                        print('-' * 10)
                        self.move_servo(self.s1_ch, col)         # Pan サーボ制御
                except Exception as e:
                    print('run:{0}'.format(e))
        except Exception as e:
            print('run:{0}'.format(e))

    # ========================================== #
    # テキストファイルの内容を全て読込む
    # afile  : ファイルパス
    # 戻り値 : テキストファイルの内容
    # ========================================== #
    def read_text(self, afile):
        result = ''
        try:
            ii = sum(1 for line in open(afile))          # ファイル行数取得
            if ii < 1: return result

            afile = afile.strip()
            if six.PY2:                                  # Python2
                with open(afile) as f:
                    alines = f.readlines()
            else:                                        # Python3
                with open(afile, encoding='utf-8') as f:
                    alines = f.readlines()
            result = alines[ii - 1]                      # 最終行の内容を返す
        except Exception as e:
            print('readText:{0}'.format(e))
        return result

    # ========================================== #
    # テキストファイル書き込み
    # f_name : ファイル名
    # wdata  : 書き込みデータ
    # mode   : 書込みモード('a':追加 'w':上書き)
    # 戻り値 : 無し
    # ========================================== #
    def write_text(self, f_name, wdata, mode='w'):
        try:
            wdata = str(wdata)
            if six.PY2:
                with open(f_name, mode) as f:
                    f.write(wdata)
                    f.flush()
            if six.PY3:
                with open(f_name, mode, encoding='utf-8') as f:
                    f.write(wdata)
                    f.flush()
        except Exception as e:
            print('write_text:{0}'.format(e))

    # ========================================== #
    # grideye から出力された文字列チェック
    # val    : 解析元文字列 e.g : 3,6,32.65 最高温度位置 -> 3行,6桁,32.65℃
    # 戻り値 : row  : 最高温度行位置
    #          col  : 最高温度桁位置
    #          temp : 最高温度
    # ========================================== #
    def cmd_check(self, val):
        row = -1; col = -1; temp = -99.9
        try:
            cmds = val.split(',')
            if len(cmds) == 3:
                row  = int(cmds[0], 10)
                col  = 7 - int(cmds[1], 10) # PAN モーター移動方向制御
                temp = float(cmds[2])
        except Exception as e:
            print('cmd_check:{0}'.format(e))
        return row, col, temp

    # ========================================== #
    # サーボ制御
    # ch     : サーボチャネル
    # sto    : 移動位置
    # 戻り値 : 無し
    #
    # self.s0_now : サボ現在位置
    # self.s1_now : サボ現在位置
    # ========================================== #
    def move_servo(self, ch, sto):
        try:
            if ch == self.s0_ch:   # Tilt サーボ
                sto = sto if sto >= self.s0_Min else self.s0_Min
                sto = sto if sto <= self.s0_Max else self.s0_Max
                self.servo.servo_between(self.s0_ch, self.s0_now, sto, self.s0_Min, self.s0_Max, self.m_wait)
                self.s0_now = sto
            elif ch == self.s1_ch: # Pan サーボ
                sto = sto if sto >= self.s1_Min else self.s1_Min
                sto = sto if sto <= self.s1_Max else self.s1_Max
                self.servo.servo_between(self.s1_ch, self.s1_now, sto, self.s1_Min, self.s1_Max, self.m_wait)
                self.s1_now = sto
        except Exception as e:
            print('move_servo:{0}'.format(e))

# end class MoveServo

# ============================================== #
# メインループ
# ============================================== #
def loop():
    try:
        ms = MoveServo(fwait=1.0, fname='a.txt', addr=0x4f, frq=50) # サーボ駆動クラス
        ms.start()
        while True: time.sleep(60)
    except KeyboardInterrupt as e:
        ms.servo.servo_between(ms.s0_ch, ms.s0_now, ms.s0_Center, ms.s0_Min, ms.s0_Max, ms.m_wait)
        ms.servo.servo_between(ms.s1_ch, ms.s1_now, ms.s1_Center, ms.s1_Min, ms.s1_Max, ms.m_wait)
        raise
    except Exception as e:
        print('loop:{0}'.format(e))

# ============================================== #
if __name__=='__main__':
    loop()
