#!/usr/bin/python
# -*- coding: UTF-8 -*-

'''
  File Name : moo_servo.py
  開発環境  : Raspberry Pi JESSIE
              Python 3.4.2
              Python 2.7.9
              ROBOBA051(Raspberry Pi 拡張ボード)

  ROBOBA045 : PCA9685(PWMコントローラ)サーボモーター制御クラス

  Adafruit_PWM_Servo_Driver ライブラリ使用
  https://learn.adafruit.com/adafruit-16-channel-servo-driver-with-raspberry-pi/using-the-adafruit-library

  cd ~
  git clone https://github.com/adafruit/Adafruit_Python_PCA9685.git
  cd Adafruit_Python_PCA9685
  sudo python3 setup.py install --record install_files3.txt  <- Python3
  sudo python setup.py install --record install_files.txt    <- Python2

  CopyLeft 2016 Moo Soft (Akiyosi Yokoyama)

  ===========================================
  SG90 9g Micro Servo
    50 Hz(20mS周期)
    Position "0"   (0.5 ms pulse)
             "90"  (1.5 ms pulse)
             "180" (2.4 ms pulse)
  データシート : http://akizukidenshi.com/download/ds/towerpro/SG90.pdf
  ===========================================
'''
import Adafruit_PCA9685
import time

# ============================================== #
# サーボモーター制御クラス
# ============================================== #
class MooServo(object):

    # ========================================== #
    # コンストラクタ
    # addr : PCA9685 アドレス
    # frq  : 周波数
    # ========================================== #
    def __init__(self, addr=0x4f, frq=50):
        self.pwm = Adafruit_PCA9685.PCA9685(addr) # PCA9685 PWM 制御モジュールインスタンス
        self.pwm.set_pwm_freq(frq)                # 周波数設定 Freq:40～1000

    # ========================================== #
    # 開始値、終了値を指定してサーボ駆動
    # ch     : サーボチャネル
    # start  : 開始値
    # end    : 終了値
    # s_Min  : 設定できる最小値
    # e_Max  : 設定できる最大値
    # wait   : 待ち時間
    # 戻り値 : 無し
    # ========================================== #
    def servo_between(self, ch, start, end, sMin, eMax, wait):
        try:
            start = start if start >= sMin else sMin # 引数チェック
            end   = end   if end   <= eMax else eMax # 引数チェック
            direc = 1 if end - start > 0 else -1     # 移動方向
            self.pwm.set_pwm(ch, 0, start)           # 開始位置に移動
            for val in range(start, end, direc):
                time.sleep(wait)
                self.pwm.set_pwm(ch, 0, val)
        except Exception as e:
            print('servo_between:{0}'.format(e))

    # ========================================== #
    # 開始値、終了値を指定して全てのサーボ駆動
    # start  : 開始値
    # end    : 終了値
    # s_Min  : 設定できる最小値
    # e_Max  : 設定できる最大値
    # wait   : 待ち時間
    # 戻り値 : 無し
    # ========================================== #
    def servo_between_all(self, start, end, sMin, eMax, wait):
        try:
            start = start if start >= sMin else sMin # 引数チェック
            end   = end   if end   <= eMax else eMax # 引数チェック
            direc = 1 if end - start > 0 else -1     # 移動方向
            self.pwm.setAllPWM(0, start)             # 開始位置に移動
            for val in range(start, end, direc):
                time.sleep(wait)
                self.pwm.setAllPWM(0, val)
        except Exception as e:
            print('servo_between_all:{0}'.format(e))

    # ========================================== #
    # サーボ駆動
    # ch     : サーボチャネル
    # val    : 設定値
    # 戻り値 : 無し
    # ========================================== #
    def set_servo(self, ch, val):
        try:
            self.pwm.set_pwm(ch, 0, val)
        except Exception as e:
            print('set_servo:{0}'.format(e))

    # ========================================== #
    # 全てのサーボ駆動
    # val    : 設定値
    # 戻り値 : 無し
    # ========================================== #
    def set_servo_all(self, val):
        try:
            self.pwm.setAllPWM(0, val)
        except Exception as e:
            print('set_servo_all:{0}'.format(e))

    # ========================================== #
    # パルスから set_pwm(ch, on, off) の
    # 引数 off の値計算(サーボモーターの角度)
    # pulse  : パルス
    # hz     : 周波数
    # val    : 表示文字列
    # 戻り値 : set_pwm(ch, on, off) の off の計算値
    # ========================================== #
    def get_pulse(self, pulse, hz, val):
        result = None
        try:
            pulseLength  = 1000000 # 1,000,000 us per second
            pulseLength /= hz      # 周波数(Hz) print("%d us per period" % pulseLength)
            pulseLength /= 4096    # 12 bits of resolution print("%d us per bit" % pulseLength)
            pulse *= 1000
            pulse /= pulseLength
            print('{0} -> float pulse:{1}'.format(val, pulse))
            result = int(round(pulse, 0))
            print('{0} -> int   pulse:{1}'.format(val, result))
        except Exception as e:
            print('get_pulse:{0}'.format(e))
        return result

# ============================================== #
# メインループ
# ============================================== #
def loop():
    s0_ch   = 0    # サーボ0 チャネル
    s1_ch   = 1    # サーボ1 チャネル
    pwm_frq = 50   # 周波数
    pcaaddr = 0x4f # PCA9685 スレーブアドレス
    try:
        servo     = MooServo(addr=pcaaddr, frq=pwm_frq)
        s_Min     = servo.get_pulse(0.5, pwm_frq, 'SG90 9g Micro Servo : 0.5ms ->   0')
        s_Center  = servo.get_pulse(1.5, pwm_frq, 'SG90 9g Micro Servo : 1.5ms ->  90')
        s_Max     = servo.get_pulse(2.4, pwm_frq, 'SG90 9g Micro Servo : 2.4ms -> 180')
        s0_Min    = s_Min + 11
        s0_Center = s_Center
        s0_Max    = s_Max - 11
        s1_Min    = s_Min + 11
        s1_Center = s_Center
        s1_Max    = s_Max - 11
        while True:
            m_wait = 0.003
            servo.servo_between(s0_ch, s0_Center, s0_Max, s0_Min, s0_Max, m_wait)
            servo.servo_between(s1_ch, s1_Center, s1_Max, s1_Min, s1_Max, m_wait)
            time.sleep(1)
            servo.servo_between(s0_ch, s0_Max, s0_Min, s0_Min, s0_Max, m_wait)
            servo.servo_between(s1_ch, s1_Max, s1_Min, s1_Min, s1_Max, m_wait)
            time.sleep(1)
            servo.servo_between(s0_ch, s0_Min, s0_Center, s0_Min, s0_Max, m_wait)
            servo.servo_between(s1_ch, s1_Min, s1_Center, s1_Min, s1_Max, m_wait)
            time.sleep(1)
    except KeyboardInterrupt:
        servo.set_servo(s0_ch, s0_Center)
        servo.set_servo(s1_ch, s1_Center)

# ============================================== #
if __name__=='__main__':
    loop()
